#include <mes2.h>
#include <h8/reg3067.h>

#define NAK	SDA
#define ACK	0
#define SDA	0x40
#define SCL	0x80
#define DREG	PADR
#define	CREG	PADDR

static void _wait() {
	volatile int w;
	w = 0;
}

static void start_i2c() {
	CREG = SCL + SDA;
	DREG &= ~SCL;
	DREG |= SDA;
	_wait();
	DREG |= SCL;
	DREG &= ~SDA;
	_wait();
	DREG &= ~SCL;
	CREG = SCL;
}

static void stop_i2c() {
	CREG = SCL + SDA;
	DREG &= ~SCL;
	DREG &= ~SDA;
	_wait();
	DREG |= SCL;
	DREG |= SDA;
	_wait();
	DREG &= ~SCL;
	CREG = SCL;
}

static int writebyte(int data) {
	int	i, mask;

	CREG = SCL + SDA;
	mask = 0x80;
	for(i = 0;i < 8;i++) {
		if(data & mask) DREG |= SDA;
		else DREG &= ~SDA;
		DREG |= SCL;
		_wait();
		DREG &= ~SCL;
		mask >>= 1;
		_wait();
	}
	CREG = SCL;
	DREG |= SCL;
	_wait();
	i = DREG & SDA;
	DREG &= ~SCL;
	return i;
}

static int readbyte() {
	int	i, data;

	data = 0;
	for(i = 0;i < 8;i++) {
		data <<= 1;
		DREG |= SCL;
		if(DREG & SDA) data++;
		_wait();
		DREG &= ~SCL;
		_wait();
	}
	return data;
}

static void ack() {
	CREG = SCL + SDA;
	DREG &= ~SDA;
	DREG |= SCL;
	_wait();
	DREG &= ~SCL;
	CREG = SCL;
}

static void nak() {
	CREG = SCL + SDA;
	DREG |= SDA;
	DREG |= SCL;
	_wait();
	DREG &= ~SCL;
	DREG &= ~SDA;
	CREG = SCL;
}

static int status() {
	return DREG & SDA;	
}

static int send_ctrl(int ctrl) {
	int	ret, count;

	count = 0;
	do {
		start_i2c();
		ret = writebyte(ctrl);
		if(count++ > 1000) return -1;
	} while(ret == NAK);
	return 0;
}

static void i2c_init() {
	CREG = SCL;
	DREG = 0;
}

#define RTC_WRITE	0xa2
#define RTC_READ	0xa3

static int rtc_read(int addr) {
	int	data, ret;

	ret = send_ctrl(RTC_WRITE);
	if(ret == -1) return -1;
	writebyte(addr);
	ret = send_ctrl(RTC_READ);
	if(ret == -1) return -1;
	data = readbyte();
	nak();
	stop_i2c();
	return data;
}

static int rtc_write(int addr, int data) {
	int	ret;

	ret = send_ctrl(RTC_WRITE);
	if(ret == -1) return -1;
	ret = writebyte(addr);
	ret = writebyte(data);
	stop_i2c();
	return 0;
}

static int get_value(int reg) {
	int	data;

	data = reg & 0x0f;
	if(reg & 0x10) data += 10;
	if(reg & 0x20) data += 20;
	if(reg & 0x40) data += 40;
	if(reg & 0x80) data += 80;
	return data;
}

static int set_value(int value) {
	int	reg, data;

	reg = (value / 10) << 4;
	data = value % 10;
	reg += data;
	return reg;
}

int main(int argc, char **argv) {
	int	year, month, week, day, hour, min, sec;
	int	i, value;

	i2c_init();
	if(argc == 3 && argv[1][0] == '-') {
		rtc_write(0, 0);
		sscanf(argv[2], "%d", &value);
		switch(argv[1][1]) {
		case 'Y':
			rtc_write(8, set_value(value));
			break;
		case 'M':
			rtc_write(7, set_value(value));
			break;
		case 'D':
			rtc_write(5, set_value(value));
			break;
		case 'h':
			rtc_write(4, set_value(value));
			break;
		case 'm':
			rtc_write(3, set_value(value));
			break;
		case 's':
			rtc_write(2, set_value(value));
			break;
		}
	} else {
		for(i = 0;i < 20;i++) {
			year = get_value(rtc_read(8));
			month = get_value(rtc_read(7) & 0x1f);
			week = get_value(rtc_read(6) & 0x07);
			day = get_value(rtc_read(5) & 0x3f);
			hour = get_value(rtc_read(4) & 0x3f);
			min = get_value(rtc_read(3) & 0x7f);
			sec = get_value(rtc_read(2) & 0x7f);
			printf("20%02d/%d/%d  ", year, month, day);
			printf("%d:%02d:%02d\r", hour, min, sec);
			sleep(1000);
		}
	}
}
